/***************************************************************************/
/* bs_compare.c								   */
/*									   */
/* Function definition and comparison values for the string compare fcn.   */
/***************************************************************************/

#include "bs_compare.h"		/* get our defs */

/***************************************************************************/
/* bs_compare								   */
/*									   */
/* Compare two byte strings, a and b, and return GREATER, EQUAL, LESSER    */
/* or INCOMPARABLE as appropriate. 					   */
/***************************************************************************/

int bs_compare( pBytes a, pBytes b )
{
  if ((a == NULL) || (b == NULL)) return ( INCOMPARABLE ) ;
  /* check the display hints */
  if (a->d_lth != b->d_lth) return ( INCOMPARABLE ) ;
  if (a->d_lth > 0)		/* if these have display hints */
    if (memcmp( a->disp, b->disp, a->d_lth ) != 0)
      return ( INCOMPARABLE ) ;	/* they must have the same display hints */
  return ( compare2( a->val, a->v_lth, b->val, b->v_lth ) ) ;
} /* bs_compare */

/***************************************************************************/
/* bin_compare								   */
/*									   */
/* Compare two byte strings, a and b, as unsigned binary -- therefore	   */
/* right justified.							   */
/***************************************************************************/

int bin_compare( pBytes a, pBytes b )
{
  if ((a == NULL) || (b == NULL)) return ( INCOMPARABLE ) ;
  /* check the display hints */
  if (a->d_lth != b->d_lth) return ( INCOMPARABLE ) ;
  if (a->d_lth > 0)		/* if these have display hints */
    if (memcmp( a->disp, b->disp, a->d_lth ) != 0)
      return ( INCOMPARABLE ) ;	/* they must have the same display hints */
  return ( compare3( a->val, a->v_lth, b->val, b->v_lth ) ) ;
} /* bin_compare */

/***************************************************************************/
/* num_compare								   */
/*									   */
/* Compare two byte strings, a and b, as signed ascii numbers, right	   */
/* justified. 								   */
/***************************************************************************/

int num_compare( pBytes a, pBytes b )
{
  if ((a == NULL) || (b == NULL)) return ( INCOMPARABLE ) ;
  /* check the display hints */
  if (a->d_lth != b->d_lth) return ( INCOMPARABLE ) ;
  if (a->d_lth > 0)		/* if these have display hints */
    if (memcmp( a->disp, b->disp, a->d_lth ) != 0)
      return ( INCOMPARABLE ) ;	/* they must have the same display hints */
  return ( compare4( a->val, a->v_lth, b->val, b->v_lth ) ) ;
} /* num_compare */

/***************************************************************************/
/* bs_max								   */
/*									   */
/* Return the max of a and b.  If either is NULL, the other is the max.	   */
/* This is consistent with date comparison rules.			   */
/***************************************************************************/

pBytes bs_max( pBytes a, pBytes b )
{
  if (a == NULL) return ( b ) ;
  if (b == NULL) return ( a ) ;
  if (bs_compare( a, b ) == GREATER)
    return ( a ) ;
  else
    return ( b ) ;
} /* bs_max */

/***************************************************************************/
/* bs_min								   */
/*									   */
/* Return the min of a and b.  If either is NULL, the other is the min.	   */
/* This is consistent with date comparison rules.			   */
/***************************************************************************/

pBytes bs_min( pBytes a, pBytes b )
{
  if (a == NULL) return ( b ) ;
  if (b == NULL) return ( a ) ;
  if (bs_compare( a, b ) == LESSER)
    return ( a ) ;
  else
    return ( b ) ;
} /* bs_min */
