/***************************************************************************/
/* bs_compare.h								   */
/*									   */
/* Function definition and comparison values for the string compare fcn.   */
/***************************************************************************/

#if !defined( __BS_COMPARE_H__ )
#define __BS_COMPARE_H__

#include "compare.h"		/* get defs of GREATER, etc. */
#include "sexp.h"		/* S-expression structures */

/***************************************************************************/
/* bs_compare								   */
/*									   */
/* Compare two byte strings, a and b, and return GREATER, EQUAL, LESSER    */
/* or INCOMPARABLE as appropriate. 					   */
/***************************************************************************/

int bs_compare( pBytes a, pBytes b ) ;

/***************************************************************************/
/* bs_max								   */
/*									   */
/* Return the max of a and b.  If either is NULL, the other is the max.	   */
/* This is consistent with date comparison rules.			   */
/***************************************************************************/

pBytes bs_max( pBytes a, pBytes b ) ;

/***************************************************************************/
/* bs_min								   */
/*									   */
/* Return the min of a and b.  If either is NULL, the other is the min.	   */
/* This is consistent with date comparison rules.			   */
/***************************************************************************/

pBytes bs_min( pBytes a, pBytes b ) ;

/***************************************************************************/
/* num_compare								   */
/*									   */
/* Compare two byte strings, a and b, as signed ascii numbers, right	   */
/* justified. 								   */
/***************************************************************************/

int num_compare( pBytes a, pBytes b ) ;

/***************************************************************************/
/* bin_compare								   */
/*									   */
/* Compare two byte strings, a and b, as unsigned binary -- therefore	   */
/* right justified.							   */
/***************************************************************************/

int bin_compare( pBytes a, pBytes b ) ;

#endif /* __BS_COMPARE_H__ */
