#include "compare.h"

#ifndef NULL
#define NULL ((void *) 0)
#endif

/***************************************************************************/
/* compare								   */
/*									   */
/* Compare two C strings, for greater, lesser or equal status.		   */
/* Comparison is case sensitive.					   */
/*									   */
/***************************************************************************/

int compare( char *a, int a_lth , char *b )
{
  unsigned char c, d ;

  if (a == b) return ( EQUAL ) ;
  if ((a == NULL) || (b == NULL)) return ( INCOMPARABLE ) ;

  do {				/* keep looping while they're equal */
    if ((a_lth--) > 0)
      c = *(a++) ;		/* fetch character */
    else
      c = '\0' ;		/* or end of string */
    d = *(b++) ;		/* fetch character or \0 */
    if (c < d) return ( LESSER ) ;
    else if (c > d) return ( GREATER ) ;
  } while (c != '\0') ;
  return ( EQUAL ) ;		/* both strings ended and were equal */
} /* compare */

/***************************************************************************/
/* compare2								   */
/*									   */
/* Compare two byte strings, for greater, lesser or equal status.	   */
/*									   */
/***************************************************************************/

int compare2( char *a, int a_lth , char *b, int b_lth )
{
  unsigned char c, d ;
  if ((a == NULL) || (b == NULL)) return ( INCOMPARABLE ) ;

  do {				/* keep looping while they're equal */
    if ((a_lth--) > 0)
      c = *(a++) ;		/* fetch character */
    else
      c = '\0' ;		/* or end of string */
    if ((b_lth--) > 0)
      d = *(b++) ;		/* fetch character */
    else
      d = '\0' ;		/* or \0 */
    if (c < d) return ( LESSER ) ;
    else if (c > d) return ( GREATER ) ;
  } while (c != '\0') ;
  return ( EQUAL ) ;		/* both strings ended and were equal */
} /* compare2 */

/***************************************************************************/
/* compare_right 							   */
/*									   */
/* Compare two unsigned right-justified strings. 			   */
/* zero is either 0 or '0', depending on whether these are binary	   */
/* or ASCII numeric strings.						   */
/***************************************************************************/

static int compare_right( char *a, int a_lth , char *b, int b_lth, int zero )
{
  unsigned char c, d ;
  if ((a == NULL) || (b == NULL)) return ( INCOMPARABLE ) ;

  while (a_lth > b_lth) {
    c = *(a++) ;
    a_lth-- ;
    if (c != zero) return ( GREATER ) ;
  } /* while a > b */
  while (b_lth > a_lth) {
    d = *(b++) ;
    b_lth-- ;
    if (d != zero) return ( LESSER ) ;
  } /* while b > a */
  return ( compare2( a, a_lth, b, b_lth ) ) ;
} /* compare_right */

/***************************************************************************/
/* compare3								   */
/*									   */
/* Compare two binary unsigned right-justified strings.			   */
/***************************************************************************/

int compare3( char *a, int a_lth , char *b, int b_lth )
{
  return ( compare_right( a, a_lth, b, b_lth, 0 ) ) ;
} /* compare3 */

/***************************************************************************/
/* compare4								   */
/*									   */
/* Compare two signed ascii numeric strings. 				   */
/***************************************************************************/

int compare4( char *a, int a_lth , char *b, int b_lth )
{
  int res ;			/* result of comparison */
  char aneg = 0 ;		/* is a negative? */
  char bneg = 0 ;		/* is b negative? */

  if ((a == NULL) || (b == NULL)) return ( INCOMPARABLE ) ;

  /* pull off any sign characters */
  switch (*a) {
  case '-': aneg = 1 ;		/* a is negative */
  case '+': a++ ;		/* gobble the character */
    a_lth-- ;			/* and drop the length */
  default:    
  } /* switch *a */

  switch (*b) {
  case '-': bneg = 1 ;		/* b is negative */
  case '+': b++ ;		/* gobble up the sign character */
    b_lth-- ;
  default:
  } /* switch *b */

  if (aneg != bneg) {
    if (aneg) return ( LESSER ) ;
    else return ( GREATER ) ;
  } /* if different signs */

  /* same sign */
  res = compare_right( a, a_lth, b, b_lth, '0' ) ;
  if (!aneg) return ( res ) ;
  /* both are negative -- reverse the sense of res */
  switch (res) {
  case LESSER: return ( GREATER ) ;
  case GREATER: return ( LESSER ) ;
  default:
  } /* switch (res) */
  return ( res ) ;
} /* compare4 */
