/***************************************************************************/
/* dumpsexp.c								   */
/*									   */
/* Debug routine to dump the parsed S-expression to f. 			   */
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "lookup.h"
#include "abbrev.h"
#include "sexp.h"
#include "dumpsexp.h"
#include "wordtab.h"
#include "cod64.h"

/***************************************************************************/
/* psp									   */
/*									   */
/* put spaces at start of line.						   */
/***************************************************************************/

static void psp( FILE *f, int n )
{
  while (n > 7) {
    fputc( '\t', f ) ;
    n -= 8 ;
  }
  while (n > 0) {
    fputc( ' ', f ) ;
    n-- ;
  }
} /* psp */

/***************************************************************************/
/* p_bs									   */
/*									   */
/* Print a byte string in hex, if outside normal range.			   */
/* Keep it under 75 columns.						   */
/***************************************************************************/

void p_bs( FILE *f, UCP b, int lth, int n )
{
  unsigned char c ;
  int i ;
  int llth ;
  
  psp( f, n ) ;
  
  llth = n ;		/* the n spaces so far */
  for (i=0; i<lth; i++) {
    if (llth > 70) {
      fputc( '\n', f ) ;
      psp( f, n ) ;
      llth = n ;
    }
    c = b[i] ;
    if ( (' ' < c) && (c < 127) ) {
      fputc( c, f ) ;
      llth++ ;
      if (c == '\\') {
	fputc( c, f ) ;
	llth++ ;
      }
    } else {
      fprintf( f, "\\%02x", c ) ;
      llth += 3 ;
    } /* else */
  }	/* for */
  fputc( '\n', f ) ;
} /* p_bs */

/***************************************************************************/
/* d_bytes								   */
/*									   */
/* Dump a byte string, b, with no end of line.				   */
/***************************************************************************/

void d_bytes( FILE *f, pBytes b, int n )
{
  if (b == NULL) {
    psp( f, n ) ;
    fputs( "(null)\n", f ) ;
  } else {
    psp( f, n ) ;
    fprintf( f, "{%d} {drop=%d} {lu=%d} {dlth=%d} {lth=%d}\n",
	     b->refcnt, b->drop_cont, b->lu, b->d_lth, b->v_lth ) ;
    if (b->d_lth > 0) {
      psp( f, n ) ;
      fputs( "[\n", f ) ;
      p_bs( f, b->disp, b->d_lth, n+2 ) ;
      psp( f, n ) ;
      fputs( "]\n", f ) ;
    }
    p_bs( f, b->val, b->v_lth, n ) ;
  } /* else */
} /* d_bytes */

/***************************************************************************/
/* d_sexp								   */
/*									   */
/* Dump the S-expression, s, with indentation, ind.			   */
/***************************************************************************/

void d_sexp( FILE *f, pSexp s, int ind )
{
  psp( f, ind ) ;
  fputs( "(\n", f ) ;
  ind += 2 ;
  if (s == NULL) {
      psp( f, ind ) ;

      fputs( "null\n", f ) ;
  } else
    while (s != NULL) {
      psp( f, ind ) ;
      fprintf( f, "(%d) (lth=%d) ", s->refcnt, s->explth ) ;
      switch (s->typ) {
      case S_KW: 
	fprintf( f, "Keyword %d: \"%s\"\n", s->u.k, spell[ s->u.k].val ) ;
	break ;
      case S_UKW: 
	fputs( "Unknown keyword \n", f ) ;
	d_bytes( f, s->u.b, ind+2 ) ;
	break ;
      case S_BS: 
	fputs( "Byte string:\n", f ) ;
	d_bytes( f, s->u.b, ind+2 ) ;
	break ;
      case S_SEXP:
	fputs( "Subexpression:\n", f ) ;
	d_sexp( f, s->u.s, ind+2 ) ;
	break ;
      } /* switch */
      s = s->term ;
    } /* while */
  ind -= 2 ;
  psp( f, ind ) ;
  fputs( ")\n", f ) ;
} /* d_sexp */

static unsigned char ptab[256] = /* text chars needing encoding */
{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,0,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} ;

static unsigned char etab[256] = /* element name chars needing encoding */
{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,
1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} ;

/***************************************************************************/
/* encode_q								   */
/*									   */
/* Scan a byte string to see if it needs to be encoded.			   */
/***************************************************************************/

int encode_q( unsigned char *buf, int blth, int param )
{
  unsigned char *tab
    = (param > 0)? ptab : etab ; /* parameter or element name? */
  int i ;			/* loop variable */

  for (i=0; i<blth; i++)
    if (tab[ buf[ i ] ]) return ( 1 ) ;
  return ( 0 ) ;		/* no encoding necessary */
} /* encode_q */

/***************************************************************************/
/* tenclth								   */
/*									   */
/* Compute the text encoding length of the given text.			   */
/***************************************************************************/

static int tenclth( unsigned char *t, int l )
{
  int lth = 0 ;			/* answer */

  if (t == NULL) return ( 0 ) ;
  while (l-- > 0)
    if (ptab[ (int) *(t++) ])
      lth += 6 ;
    else
      lth++ ;
  return ( lth ) ;
} /* tenclth */

/***************************************************************************/
/* benclth								   */
/*									   */
/* Compute the approximate base64 encoding length of the given text. 	   */
/***************************************************************************/

static int benclth( int l )
{
  return ( 18 + 4 * ((l + 2) / 3 ) ) ;
} /* benclth */

/***************************************************************************/
/* xml_bytes								   */
/*									   */
/* Dump a byte string, b, as XML for parameter k, with indentation ind.    */
/***************************************************************************/

static void xml_bytes( FILE *f, pBytes b, int ind, int k )
{
  int i ;			/* loop variable */
  int enc = 0 ;			/* boolean: do we encode this? */
  int closexs = 0 ;		/* set if we need to close an <xs:p> */
  char c ;			/* single character from b */
  int hint = 0 ;		/* is there a display hint? */
  int dob64 = 0 ;		/* do base64 encoding */

  if (b == NULL) return ;
  enc = encode_q( b->val, b->v_lth, k ) ; /* does this string need encoding? */
  hint = b->d_lth > 0 ;		/* is there a hint? */

  if ((k == -2)&&(enc || hint)) { /* do it recursively */
    xml_bytes( f, b, ind, 0 ) ; /* open separately */
    xml_bytes( f, b, ind, -1 ) ; /* close separately */
    return ;			/* and return */
  } /* if */
  /* OK, do more normal processing */
  psp( f, ind ) ;
  switch (k) {
  case -2:
    /* no enc or hint -- handled above */
    fputc( '<', f ) ;		/* open the tag */
    fwrite( b->val, 1, b->v_lth, f ) ;
    fputs( "/>\n", f ) ;
    return ;

  case -1:
    if (enc || hint)
      fputs( "</xs:custom>\n", f ) ;
    else {
      fputs( "</", f ) ;
      fwrite( b->val, 1, b->v_lth, f ) ;
      fputs( ">\n", f ) ;
    } /* else */
    return ;

  case 0:	/* element open and close */
    if (enc || hint) {
      fputs( "<xs:custom>\n", f ) ;
      ind += 2 ;		/* bump indentation temporarily */
      psp( f, ind ) ;		/* and indent to there */
      /* and fall through into case 1 */
    } else {
      fputc( '<', f ) ;		/* start the element */
      fwrite( b->val, 1, b->v_lth, f ) ;
      fputs( ">\n", f ) ;
      return ;
    } /* else */
    /* maybe falling through */

  case -3:			/* only parameter, special case */
    if (!(enc || hint))		/* if no encode or display hint */
      break ;			/* it needs no <xs:p> element */
    /* otherwise, fall through and put out the <xs:p> */
  case 1:			/* first parameter of several */
  default:			/* other parameters */
    fputs( "<xs:p", f ) ;	/* note the param */
    closexs = 1 ;		/* close it on exit */
    /* is there a display type? */
    if (b->d_lth > 0) {		/* note hint if any */
      fputs( " mime-type=\"", f ) ;
      fwrite( b->disp, 1, b->d_lth, f ) ;
      fputc( '\"', f ) ;
    } /* display type */
    if (enc)
      if ((dob64=(tenclth( b->val, b->v_lth ) > benclth( b->v_lth ))))
	fputs( " encoding=\"base64\"", f ) ;
    fputs( "> ", f ) ;
  } /* switch (k) */

  if (dob64) {			/* have to encode this parameter */
    char *b64b = NULL ;		/* base64 buffer */
    int blth = 16 + ((4*b->v_lth)/3) ;
    long olth = 0 ;
    int wid = 72 - ind ;

    if (wid < 16) wid = 16 ;
    blth = blth / wid ;		/* how many lines? */
    blth = (blth + 2) * (wid + ind + 8) ; /* how many bytes (plus slop)? */
    b64b = malloc( blth ) ;
    if (b64b == NULL)
      fputs( "(memory error)", f ) ;
    else {

      if (wid < 16) wid = 16 ;
      fputc( '\n', f ) ;
      psp( f, ind+2 ) ;
      enc64( b64b, blth, &olth, b->val, b->v_lth, wid, ind+2 ) ;
      fwrite( b64b, 1, olth, f ) ;
      psp( f, ind-1 ) ;
    } /* else */
    free( b64b ) ;
  } else
    if (enc) {			/* individual character encoding */
      for (i=0; i<b->v_lth; i++ ) {
	c = b->val[i] ;
	if (ptab[ (int) c ])
	  fprintf( f, "&#x%02x;", c ) ;
	else
	  fputc( c, f ) ;
      }	/* for i */
    } else
      fwrite( b->val, 1, b->v_lth, f ) ; /* no encoding */

  if (closexs)
    fputs( " </xs:p>\n", f ) ;
  else
    fputc( '\n', f ) ;

  return ;
} /* xml_bytes */

/***************************************************************************/
/* xml_sexp								   */
/*									   */
/* Dump the S-expression, s, with indentation, ind.			   */
/***************************************************************************/

void xml_sexp( FILE *f, pSexp s, int ind )
{
  int param = 0 ;		/* parameter number */
  pSexp typ = NULL ;		/* the type node */
  char *kw ;			/* keyword spelling */

  if (s == NULL) return ;	/* no expression -> no action */

  while (s != NULL) {
    switch (s->typ) {
    case S_KW:
      typ = s ;			/* note the type node */
      psp( f, ind ) ;
      if (s->u.k == O__STAR_)
	kw = "xs:star" ;
      else
	kw = spell[ s->u.k ].val ;
      if (s->term == NULL) {
	fprintf( f, "<%s/>\n", kw ) ;
	return ;
      } else
	fprintf( f, "<%s>\n", kw ) ;
      break ;
    case S_UKW:
      typ = s ;			/* note the type node */
      if (s->term == NULL) {
	xml_bytes( f, s->u.b, ind, -2 ) ; /* open the element */
	return ;
      } else
	xml_bytes( f, s->u.b, ind, 0 ) ; /* open the element */
      break ;
    case S_BS: 
      if ((s->term == NULL) && (param == 1))
	xml_bytes( f, s->u.b, ind+2, -3 ) ;
      else
	xml_bytes( f, s->u.b, ind+2, param ) ;
      break ;
    case S_SEXP:
      xml_sexp( f, s->u.s, ind+2 ) ;
      break ;
    } /* switch */
    s = s->term ;		/* go to the next list element */
    param++ ;			/* count the param number */
  } /* while */
  if (typ == NULL) return ;
  switch (typ->typ) {
  case S_KW:
    psp( f, ind ) ;
    fprintf( f, "</%s>\n", kw ) ;
    break ;
  case S_UKW:
    xml_bytes( f, typ->u.b, ind, -1 ) ; /* close this element */
  default:
    break ;
  } /* switch (typ->typ) */
  return ;
} /* xml_sexp */
