/***************************************************************************/
/* psexp.h								   */
/*									   */
/*  Routines for parsing S-expressions.					   */
/***************************************************************************/

#if !defined( __PSEXP_H__ )
#define __PSEXP_H__

#include "free.h"
#include "strread.h"

/***************************************************************************/
/* p_lth								   */
/*									   */
/* Read the length field from the current string reader.		   */
/***************************************************************************/

int p_lth( ULP pRes, pSreader s ) ;

/***************************************************************************/
/* p_bytes								   */
/*									   */
/* Parse a byte string from the current string reader and pack it into	   */
/* a new byte string block.						   */
/***************************************************************************/

int p_bytes( pBytes *ppb, pSreader s ) ;

/***************************************************************************/
/* p_sexp								   */
/*									   */
/* Parse an S-expression from a string block.				   */
/* An S-expression is enclosed in "()" and starts with a byte string.	   */
/* Look up the byte string it starts with.				   */
/***************************************************************************/

int p_sexp( pSexp *pps, pSreader s ) ;

/***************************************************************************/
/* kill_expb								   */
/*									   */
/* Walk s recursively, killing all expb and explth fields.		   */
/* This expression started from a contiguous block (e.g. a literal string) */
/*     but it is being edited into something non-contiguous.		   */
/***************************************************************************/

void kill_expb( pSexp s ) ;

/***************************************************************************/
/* lit_sexp								   */
/*									   */
/* Parse and return a literal S-expression.				   */
/***************************************************************************/

int lit_sexp( pSexp *pps, char *lit ) ;

#endif /* __PSEXP_H__ */
