/***************************************************************************/
/* s2x.c								   */
/*									   */
/*  parse one S-expression from stdin and dump the parse tree		   */
/*  to stdout, in XML form. 						   */
/*									   */
/***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "abbrev.h"
#include "sexp.h"
#include "strread.h"
#include "psexp.h"
#include "dumpsexp.h"

#define INB_LTH (32768)		/* input buffer length */

/***************************************************************************/
/* main									   */
/*									   */
/* Read a single S-expression from stdin and parse it.  Then dump the	   */
/* parse tree to stdout.						   */
/*									   */
/***************************************************************************/

int main( int argc, char **argv )
{
  int r ;			/* return value */
  UC x[INB_LTH] ;		/* file input */
  unsigned int xlth ;		/* length of x */
  pSexp s = NULL ;		/* the S-expression */
  Sreader rdr ;			/* the expression reader */

  xlth = fread( x, 1, INB_LTH, stdin ) ;

  if (xlth >= INB_LTH) {
    fprintf( stderr,
"Input S-expression larger than the maximum of %d bytes.\n",
	     INB_LTH ) ;
    return ( 1 ) ;
  }

  rdr.cur = rdr.top = x ;	/* start of the expression */
  rdr.end = x + xlth ;		/* end of the bytes read */
  /* the expression might end before that */

  r = p_sexp( &s, &rdr ) ;	/* parse the expression */
  if (r != 0) {
    fprintf( stderr, "S-expression parsing error # %d\n", r ) ;
    return ( 1 ) ;
  }

  /* expression parsed correctly */

  /* now, dump the parse tree */
  xml_sexp( stdout, s, 0 ) ;	/* dump s as XML */

  return (0) ;
} /* main */
