/***************************************************************************/
/* sexp.h								   */
/*									   */
/* General S-expression structure definitions for SPKI code.		   */
/*									   */
/* See free.h for an explanation of how to handle refcnt.		   */
/*									   */
/***************************************************************************/

#if !defined(__SEXP_H__)
#define __SEXP_H__

#include "abbrev.h"

/***************************************************************************/
/* Forward references to all structures defined here. 			   */
/***************************************************************************/

typedef struct _bytes_ *pBytes ; /* Byte-string block  */
typedef struct _sexp_ *pSexp ;	/* S-expression block  */

/***************************************************************************/
/* byte string structure -- with optional display hint			   */
/***************************************************************************/

typedef struct _bytes_ {	/* byte-string */
  pBytes nxt ;			/* next byte string, for free.c */
  short refcnt ;		/* reference count (drop when 0) */
  short drop_cont ;		/* non-0 means internal strings were carved */
  int lu ;			/* lookup value */
  int d_lth ;			/* length of the display hint */
  CP disp ;			/* display hint */
  int v_lth ;			/* length of the byte string itself */
  UCP val ;			/* byte string */
} Bytes ;

/***************************************************************************/
/* Types of S-expression elements					   */
/***************************************************************************/

#define S_KW (1)		/* 1st term: recognized keyword */
#define S_UKW (2)		/* 1st term: unknown keyword */
#define S_BS (3)		/* other term: general byte string */
#define S_SEXP (4)		/* other term: sub-expression */

/***************************************************************************/
/* S-expression element							   */
/***************************************************************************/

typedef struct _sexp_ {		/* general S-expression */
  pSexp nxt ;			/* next S-expression, for free.c */
  int refcnt ;			/* reference count (drop when 0) */
  pSexp term ;			/* next term in the expression */
  /* the expression bytes and length are NULL if not 1st term */
  UCP expb ;			/* expression bytes, if in contiguous memory */
  int explth ;			/* expression length, if in contig. mem. */
  int typ ;			/* type of this term */
  union {
    int k ;			/* S_KW: recognized keyword */
    pBytes b ;			/* S_UKW or S_BS: byte string  */
    pSexp s ;			/* S_SEXP: sub-expression */
  } u ;
} Sexp ;

#endif /* __SEXP_H__ */
